@@ -1,11 +1,11 @@
 /* exhaleApp.cpp - source file with main() routine for exhale application executable
- * written by C. R. Helmrich, last modified in 2021 - see License.htm for legal notices
+ * written by C. R. Helmrich, last modified in 2025 - see License.htm for legal notices
  *
  * The copyright in this software is being made available under the exhale Copyright License
  * and comes with ABSOLUTELY NO WARRANTY. This software may be subject to other third-
  * party rights, including patent rights. No such rights are granted under this License.
  *
- * Copyright (c) 2018-2021 Christian R. Helmrich, project ecodis. All rights reserved.
+ * Copyright (c) 2018-2025 Christian R. Helmrich, project ecodis. All rights reserved.
  */
 
 #include "exhaleAppPch.h"
@@ -57,10 +57,16 @@
 #define _GETCWD  getcwd
 #define _STRLEN  strlen
 
+#if 0  // change this to "#if 1" to avoid garbage text in some terminals
+#define EXHALE_TEXT_INIT  ""
+#define EXHALE_TEXT_BLUE  ""
+#define EXHALE_TEXT_PINK  ""
+#else
 #define EXHALE_TEXT_INIT  "\x1b[0m"
 #define EXHALE_TEXT_BLUE  "\x1b[36m"
 #define EXHALE_TEXT_PINK  "\x1b[35m"
 #endif
+#endif
 
 // constants, experimental macros
 #if LE_ACCURATE_CALC
@@ -319,7 +325,7 @@ int main (const int argc, char* argv[])
   if (argc <= 0) return argc; // for safety
 
   const bool readStdin = (argc == 3 || argc == 5);
-  BasicWavReader wavReader;
+  BasicWavReader wavReader(1);
   int32_t* inPcmData = nullptr;  // 24-bit WAVE audio input buffer
   int32_t* inPcmRsmp = nullptr;  // temporary buffer for resampler
   uint8_t* outAuData = nullptr;  // access unit (AU) output buffer
@@ -477,7 +483,7 @@ int main (const int argc, char* argv[])
   // check arg. list, print usage if needed
   if ((argc < 3) || (argc > 6) || (argc > 1 && argv[1][1] != 0))
   {
-    fprintf_s (stdout, " Copyright 2018-2022 C.R.Helmrich, project ecodis. See License.htm for details.\n\n");
+    fprintf_s (stdout, " Copyright 2018-2025 C.R.Helmrich, project ecodis. See License.htm for details.\n\n");
 
     fprintf_s (stdout, " This software is made available under the exhale Copyright License and comes\n");
     fprintf_s (stdout, " with ABSOLUTELY NO WARRANTY. This software may be subject to other third-party\n");
@@ -644,7 +650,7 @@ int main (const int argc, char* argv[])
 #else
   if ((wavReader.open (inFileHandle, startLength, readStdin ? LLONG_MAX : lseek (inFileHandle, 0, 2 /*SEEK_END*/)) != 0) ||
 #endif
-      (wavReader.getSampleRate () >= 1000 && wavReader.getSampleRate () < 22050 && enableSbrCoding) || (wavReader.getNumChannels () >= 7))
+      (wavReader.getSampleRate () >= 1000 && wavReader.getSampleRate () < 22050 && enableSbrCoding) || (wavReader.getNumChannels () >= (enableSbrCoding ? 3u : 7u)))
   {
     _ERROR1 (" ERROR while trying to open WAVE file: invalid or unsupported audio format!\n\n");
 
@@ -652,6 +658,10 @@ int main (const int argc, char* argv[])
     {
       _ERROR2 (" The sampling rate is %d kHz but encoding using eSBR requires at least 22 kHz.\n\n", wavReader.getSampleRate () / 1000);
     }
+    if (wavReader.getNumChannels () >= 3 && enableSbrCoding)
+    {
+      _ERROR2 (" The channel count is %d but exhale can't encode multichannel audio with eSBR.\n\n", wavReader.getNumChannels ());
+    }
     i = 8192; // return value
 
     goto mainFinish; // audio format invalid
@@ -862,7 +872,7 @@ int main (const int argc, char* argv[])
 #ifdef NO_PREROLL_DATA
     outAuData = (uint8_t*) malloc ((6144 >> 3) * numChannels); // max frame AU size
 #else
-    outAuData = (uint8_t*) malloc ((9216 >> 3) * numChannels); // max frame AU size
+    outAuData = (uint8_t*) malloc ((6144 >> 2) * numChannels); // max frame AU size
 #endif
     if ((inPcmData == nullptr) || (outAuData == nullptr))
     {
@@ -897,7 +907,7 @@ int main (const int argc, char* argv[])
       const unsigned mod3Percent = unsigned ((expectLength * (3 + (coreSbrFrameLengthIndex & 3))) >> 17);
       uint32_t byteCount = 0, bw = (numChannels < 7 ? loudStats : 0);
 #endif
-      uint32_t br, bwMax = 0; // br will be used to hold bytes read and/or bit-rate
+      uint32_t br, bwMax = 0, bwTmp = 0; // br will hold bytes read and/or bit-rate
       uint32_t headerRes = 0;
       // initialize LoudnessEstimator object
       LoudnessEstimator loudnessEst (inPcmData, 24 /*bit*/, sampleRate, numChannels);
@@ -1014,7 +1024,9 @@ int main (const int argc, char* argv[])
           goto mainFinish; // writeout error
         }
       }
-
+#if 0
+      std::cout << "\n" << "headerSizeBytes " << (headerRes - 34) << "\n";
+#endif
       i = 1; // for progress bar
 
       // resample initial frame if necessary
@@ -1061,6 +1073,7 @@ int main (const int argc, char* argv[])
         goto mainFinish; // coder-time error
       }
 #endif
+      bwTmp = bw;
 #ifdef NO_PREROLL_DATA
       if (bwMax < bw) bwMax = bw;
 
@@ -1103,7 +1116,14 @@ int main (const int argc, char* argv[])
 #endif
           goto mainFinish; // encoding error
         }
-        if (bwMax < bw) bwMax = bw;
+#ifdef NO_PREROLL_DATA
+        bw = __min ((6144/8) * numChannels, bw);
+#else
+        bw = __min (1536 * numChannels, bw);
+#endif
+        bwTmp = (enableSbrCoding ? bw : (bwTmp + bw) >> 1u);
+        if (bwMax < bwTmp) bwMax = bwTmp;
+        bwTmp = bw;
 
         // write new AU, add frame to header
 #if ENABLE_STDOUT_LOAS
@@ -1156,7 +1176,14 @@ int main (const int argc, char* argv[])
 #endif
         goto mainFinish; // coder-time error
       }
-      if (bwMax < bw) bwMax = bw;
+#ifdef NO_PREROLL_DATA
+      bw = __min ((6144/8) * numChannels, bw);
+#else
+      bw = __min (1536 * numChannels, bw);
+#endif
+      bwTmp = (enableSbrCoding ? bw : (bwTmp + bw) >> 1u);
+      if (bwMax < bwTmp) bwMax = bwTmp;
+      bwTmp = bw;
 
       // write final AU, add frame to header
 #if ENABLE_STDOUT_LOAS
@@ -1215,7 +1242,9 @@ int main (const int argc, char* argv[])
 #endif
           goto mainFinish; // encoding error
         }
-        if (bwMax < bw) bwMax = bw;
+        bwTmp = (enableSbrCoding ? bw : (bwTmp + bw) >> 1u);
+        if (bwMax < bwTmp) bwMax = bwTmp;
+        bwTmp = bw;
 
         // the flush AU, add frame to header
 #if ENABLE_STDOUT_LOAS
@@ -1319,7 +1348,7 @@ int main (const int argc, char* argv[])
       {
 #endif
       br = uint32_t (((actualLength >> 1) + 8 * (byteCount + 4 * (int64_t) mp4Writer.getFrameCount ()) * sampleRate) / actualLength);
-      bw = uint32_t (((frameLength  >> 1) + 8 * (bwMax + 4u /* maximum AU size + stsz as a bit-rate */) * sampleRate) / frameLength);
+      bw = uint32_t (((frameLength  >> 1) + 8 * (bwMax + 4u /* max. 2048 AU size + stsz as bit-rate */) * sampleRate) / frameLength);
       bw = mp4Writer.finishFile (br, bw, uint32_t (__min (UINT_MAX - startLength, actualLength)), (time (nullptr) + 2082844800) & UINT_MAX,
                                  (i == 0) && (numChannels < 7) ? outAuData : nullptr);
       // print out collected file statistics
